"""Tools for dealing with activity DAGs."""


def find_path(dag, ends, done=frozenset(), path=None, already_done=None):
    """Returns a list that includes all end points for a given DAG.

    Parameters
    ----------
    dag : dict of names to activities
        A DAG of all possible activities
    ends : set of str
        End points to compute.
    done : set of str, optional
        The nodes that have already been computed
    path : list of str, optional
        This is the return value of which activities to execute in which order.
    already_done : list of str
        The return value  of the activities that have already been computed
        that would otherwise need to be computed.

    Returns
    -------
    path : list of str
        The path through the DAG.
    already_done : list of str
        Activities that have already been computed that would otherwise need to
        be computed.
    """
    if path is None:
        path = []
    if not isinstance(done, frozenset):
        done = frozenset(done)
    if already_done is None:
        already_done = []
    pth = set(path)
    sofar = pth | done
    todo = ends - sofar
    would_do = (ends - pth) & done
    already_done.extend([x for x in sorted(would_do) if x not in already_done])
    if len(todo) == 0:
        return path, already_done
    for end in sorted(todo):
        try:
            deps = set(dag[end].deps)
        except KeyError:
            # we need to do it this way to support defaultdict
            raise KeyError('{0!r} not found in DAG!'.format(end))
        need = deps - sofar
        already_done.extend([x for x in sorted(deps & done) if x not in already_done])
        if len(need) > 0:
            find_path(dag, need, done=done, path=path, already_done=already_done)
            sofar.update(path)
        if end not in sofar:
            path.append(end)
    return path, already_done



#!/usr/bin/env python3
import os
try:
    from setuptools import setup
    HAVE_SETUPTOOLS = True
except ImportError:
    from distutils.core import setup
    HAVE_SETUPTOOLS = False


def main():
    """The main entry point."""
    with open(os.path.join(os.path.dirname(__file__), 'README.rst'), 'r') as f:
        readme = f.read()
    scripts = ['scripts/rever']
    skw = dict(
        name='re-ver',
        description='Release Versions of Software',
        long_description=readme,
        license='BSD',
        version='0.4.4',
        author='Anthony Scopatz',
        maintainer='Anthony Scopatz',
        author_email='scopatz@gmail.com',
        url='https://github.com/scopatz/rever',
        platforms='Cross Platform',
        classifiers=['Programming Language :: Python :: 3'],
        packages=['rever', 'rever.activities'],
        package_dir={'rever': 'rever', 'rever.activities': 'rever/activities'},
        package_data={'rever': ['*.xsh'], 'rever.activities': ['*.xsh']},
        scripts=scripts,
        zip_safe=False,
        install_requires=['xonsh', 'lazyasd', 'ruamel.yaml', 'github3.py'],
        python_requires=">3.4",
        )
    # WARNING!!! Do not use setuptools 'console_scripts'
    # It validates the depenendcies everytime the
    # 'rever' command is run. This validation adds ~0.2 sec. to the startup
    # time of xonsh - for every single xonsh run.  This prevents us from
    # reaching the goal of a startup time of < 0.1 sec.  So never ever write
    # the following:
    #
    #     'console_scripts': ['rever = rever.main:main'],
    #
    # END WARNING
    setup(**skw)


if __name__ == '__main__':
    main()
